// C Source File
// Created 11/21/2010; 6:34:56 AM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_bullets.h
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************


#include <tigcclib.h>
#include "extgraph.h"

#include "BasicTypes.h"
#include "BasicUtils.h"
#include "GrayUtils.h"

#include "jm_bullets.h"
#include "jm_objects.h"
#include "jm_graphics.h"
#include "jm_jumpman.h"
#include "jm_scores.h"


BULLET_OBJECT_Type bullet_list[BULLET_COUNT_MAX];
int2 bullet_count;


void FireBullet(ENEMY_OBJECT_Type* enemy, GAME_PIECE_Type bullet_type, DIR_Type direction)
{
	int2 i;
	BULLET_OBJECT_Type* bullet = &bullet_list[0];

	for (i = 0; i < bullet_count; i++)
	{
		if (bullet->delay == 0)
		{
			if ((bullet->game_piece == bullet_type) &&
			    (bullet->direction == DIR_None)) // &&
			    //(bullet->delay == 0))//der &&
			    //der ((frame_count & 3) == 0))
			{
				bullet->speed = BULLET_FIRE_SPEED;
				bullet->direction = direction;
				bullet->anim_step = 0;
				bullet->active = true;
	
				if (enemy)
				{
					bullet->x = enemy->x;
					bullet->y = enemy->y;
				}
				else
				{
					bullet->x = jumpman.x;
					bullet->y = jumpman.y + 2;
				}
	
				break;
			}
		}

		bullet++;
	}
}

void DrawBullets(void)
{
	BULLET_OBJECT_Type* bullet = &bullet_list[0];
	int2 i;
	int2 bullet_y_min = 0;

	for (i = 0; i < bullet_count; i++)
	{
		if (bullet->delay == 0)
		{
			if (bullet->game_piece == GAME_PIECE_Spear)
				bullet_y_min = -8;
				
			if ((bullet->x < 0) || (bullet->x > 160) || (bullet->y < bullet_y_min) || (bullet->y > 88))
			{
				ResetBullet(bullet);
			}
			else if (bullet->direction != DIR_None)
			{
				uint1* pSprite = (uint1*)j7 + (bullet->game_piece * (1*4*3));

				// spears are special as they have different sprites for left and right directions
				if (bullet->game_piece == GAME_PIECE_Spear)
				{
					if (bullet->direction == DIR_Left)
						pSprite += 12;
				}

				GrayClipSprite8_SMASK_R(bullet->x, bullet->y, 4, pSprite + 0, pSprite + 4, pSprite + 8, PLANE_CurrentPlane0, PLANE_CurrentPlane1);
			}
		}

		bullet++;
	}
}

void UpdateBullets(void)
{
	static const int1 spear_y_offset[18]      =  {-2,-2,-1,-1, 0,-1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 2, 2, 3}; //der may need to update this
	static const int1 hailstone_x_offset[10]  =  { 0, 1, 1, 2, 2, 1, 1, 0, 0, 0};
	static const int1 hailstone_y_offset[10]  =  {-2,-1,-1, 0, 0, 1, 1, 2, 1, 1};
	static const int1 zig_zag_xy_offset[8]    =  { 1, 1,-1,-1,-1,-1, 1, 1};
	BULLET_OBJECT_Type* bullet = &bullet_list[0];
	uint1 speed;
	uint1 direction;
	int2 i;

	for (i = 0; i < bullet_count; i++)
	{
		if (bullet->delay == 1)
		{
			if (bullet->game_piece == GAME_PIECE_Hailstone)
			{
				bullet->x = jumpman.x;
				bullet->y = 0;
				bullet->action = BULLET_ACTION_None;
			}

			bullet->delay--;
		}
		else if (bullet->delay == 0)
		{
			speed = bullet->speed;
			direction = bullet->direction;

			if (bullet->game_piece == GAME_PIECE_Hailstone)
			{
				if (bullet->action == BULLET_ACTION_None)
				{
					bullet->y += speed;
				}
				else
				{
					int1 x_adjust;

					if (bullet->action == BULLET_ACTION_Bounce_Right)
						x_adjust = 1;
					else if (bullet->action == BULLET_ACTION_Bounce_Left)
						x_adjust = -1;
					else
						x_adjust = 0;

					bullet->x += (hailstone_x_offset[bullet->anim_step] * x_adjust);
					bullet->y += (hailstone_y_offset[bullet->anim_step]);

					bullet->anim_step++;

					if (bullet->anim_step > 8) //der should this be '10'?
					{
						bullet->anim_step = 0;
						bullet->action = BULLET_ACTION_None;
					}
				}
			}
			else if (bullet->game_piece == GAME_PIECE_Spear)
			{
				if (direction == DIR_Right)
				{
					bullet->x += speed;
				}
				else if (direction == DIR_Left)
				{
					bullet->x -= speed;
				}

				bullet->y += (spear_y_offset[bullet->anim_step]);

				bullet->anim_step++;

				if (bullet->anim_step > 18)
				{
					ResetBullet(bullet);
				}
			}
			else if ((bullet->game_piece == GAME_PIECE_Zig_Zag_Bullet) && (speed == BULLET_FIRE_SPEED))
			{
				int1 offset = zig_zag_xy_offset[bullet->anim_step] * speed;

				if (direction == DIR_Up)
				{
					bullet->x += offset;
					bullet->y -= speed;
				}
				else if (direction == DIR_Down)
				{
					bullet->x += offset;
					bullet->y += speed;
				}
				else if (direction == DIR_Right)
				{
					bullet->x += speed;
					bullet->y += offset;
				}
				else if (direction == DIR_Left)
				{
					bullet->x -= speed;
					bullet->y += offset;
				}

				bullet->anim_step++;
				bullet->anim_step &= 0x7;
			}
			else //if ((speed == BULLET_NORM_SPEED) || ((bullet->game_piece == GAME_PIECE_Bullet) || (bullet->game_piece == GAME_PIECE_Jumping_Block)))
			{
				if (direction == DIR_Up)
				{
					bullet->y -= speed;
				}
				else if (direction == DIR_Down)
				{
					bullet->y += speed;
				}
				else if (direction == DIR_Right)
				{
					bullet->x += speed;
				}
				else if (direction == DIR_Left)
				{
					bullet->x -= speed;
				}
				else if (direction == DIR_UpLeft)
				{
					bullet->x -= speed;
					bullet->y -= speed;
				}
				else if (direction == DIR_UpRight)
				{
					bullet->x += speed;
					bullet->y -= speed;
				}
				else if (direction == DIR_DownLeft)
				{
					bullet->x -= speed;
					bullet->y += speed;
				}
				else if (direction == DIR_DownRight)
				{
					bullet->x += speed;
					bullet->y += speed;
				}
			}
/*

			if ((bullet->x < 0) || (bullet->x > 160) || (bullet->y < 0) || (bullet->y > 88))
			{
				ResetBullet(bullet);
			}
			else if (direction != DIR_None)
			{
//					 char buffer[16];
//					 sprintf(buffer, "Bullet %d", bullet_count);
//					 GrayUserDrawStr(64,1, buffer, COLOR_White, 0);

				//DrawPlatformPiece(bullet->game_piece, FLAG_VA, bullet->x, bullet->y);
				uint1* pSprite = (uint1*)j7 + (bullet->game_piece * (1*4*3));
				//uint1* pSprite = (uint1*)j7 + (9 * (1*4*3));
				GrayClipSprite8_SMASK_R(bullet->x, bullet->y, 4, pSprite + 0, pSprite + 4, pSprite + 8, PLANE_CurrentPlane0, PLANE_CurrentPlane1);
			}
*/
		}
		else
		{
			bullet->delay--;
		}

		bullet++;
	}
}


void CheckBullets(void)
{
	int i;
	int j;
	int k;
	BULLET_OBJECT_Type* bullet = &bullet_list[0];
	uint1  test_value;

	for (i = 0; i < bullet_count; i++)
	{
		if (bullet->delay == 0)
		{
			if ((bullet->game_piece == GAME_PIECE_Hailstone) && (bullet->action == BULLET_ACTION_None))
			{
				uint1 test_value = 0x18;
				uint1 bkgd_value;

				Sprite8Get_R(bullet->x, bullet->y + 4, 1, plane_visible_0, &bkgd_value);

				//bkgd_value = ~bkgd_value;

				if (test_value & (~bkgd_value))
				{
					if (bullet->x < (jumpman.x - 5))
						bullet->action = BULLET_ACTION_Bounce_Right;
					else if (bullet->x > (jumpman.x + 5))
						bullet->action = BULLET_ACTION_Bounce_Left;
					else
					{
						if (RandomNumber(6))
							bullet->action = BULLET_ACTION_Bounce_Up;
						else
							bullet->action = BULLET_ACTION_Bounce_Left + RandomNumber(3);
					}
				}
			}
			else if (bullet->speed == BULLET_NORM_SPEED)
			{
				if ((bullet->y >= (jumpman.y + 3)) && (bullet->y <= (jumpman.y + 5)))
				{
					if ((bullet->action >= BULLET_ACTION_Vertical) && (bullet->action <= BULLET_ACTION_HorzAndVert))
					{
						bullet->speed = BULLET_FIRE_SPEED;

						if (bullet->x > jumpman.x)
						{
							bullet->direction = DIR_Left;
						}
						else
						{
							bullet->direction = DIR_Right;
						}
					}
				}
				else if ((bullet->x >= jumpman.x) && (bullet->x <= (jumpman.x + 2)))
				{
					if ((bullet->action >= BULLET_ACTION_HorzAndVert) && (bullet->action <= BULLET_ACTION_Horizontal))
					{
						bullet->speed = BULLET_FIRE_SPEED;

						if (bullet->y > jumpman.y)
						{
							bullet->direction = DIR_Up;
						}
						else
						{
							bullet->direction = DIR_Down;
						}
					}
				}
			}

			if ((bullet->speed == BULLET_FIRE_SPEED) && (bullet->game_piece >= GAME_PIECE_Jumpman_Gunfire))
			{
				for (j = 0; j < enemy_count; j++)
				{
					ENEMY_OBJECT_Type* enemy = &enemy_list[j];

					test_value = 0;
					
					if (bullet->active)
					{
						for (k = 0; k < enemy->sprite_width[enemy->direction]; k++)
						{
							test_value |= BOUNDS_COLLIDE_UTIL(bullet->x+2, bullet->y+1, enemy->x, enemy->y, 4, 6, 2, 8);
						}
					}

					if (test_value)
					{
						if (enemy->enemy_type == GAME_PIECE_Ufo)
						{
							enemy->direction = DIR_None;
						}
						else
						{
							ResetEnemy(enemy);
						}

						ResetBullet(bullet);
						game_score += 50;
					}
				}
			}
			else if ((bullet->speed == BULLET_FIRE_SPEED) || (bullet->game_piece == GAME_PIECE_Hailstone))
			{
				test_value = 0;
				
				if (bullet->active)
				{
					test_value = BOUNDS_COLLIDE_UTIL(bullet->x+2, bullet->y+1, jumpman.x_orig, jumpman.y_orig+1, 4, 6, 2, 8);
				}

				if ((test_value) && (jumpman.action < ACTION_Fall))
				{
					if (bullet->game_piece == GAME_PIECE_Jumping_Block)
					{
						if ((jumpman.action != ACTION_Jump_Up) && (jumpman.action != ACTION_Jump))
							SetJumpmanAction(ACTION_Jump, DIR_Left + RandomNumber(2));
					}
					else if (jumpman.action < ACTION_Fall)
					{
						SetJumpmanAction(ACTION_Fall, DIR_Down);
					}
				}
			}
		}

		bullet++;
	}
}


void InitBullets(void)
{
	BULLET_OBJECT_Type* bullet = &bullet_list[0];
	GAME_PIECE_Type bullet_type = GAME_PIECE_First_Bullet;
	OBJECT_Type* object;
	bool done = false;

	bullet_count = 0;
	memset((void*)(bullet_list), 0, sizeof(BULLET_OBJECT_Type) * BULLET_COUNT_MAX);


	while ((!done) && (bullet_count < BULLET_COUNT_MAX))
	{
		object = FindObjectEntryByType(bullet_type, false);

		if (object)
		{
			bullet->game_piece = object->game_piece;
			bullet->action = object->action;

			ResetBullet(bullet);

			bullet_count++;
			bullet++;
		}
		else
		{
			bullet_type++;

			if (bullet_type > GAME_PIECE_Last_Bullet)
				done = true;
		}
	}
}


void ResetBullet(BULLET_OBJECT_Type* bullet)
{
	bullet->anim_step = 0;

	if (bullet->game_piece >= GAME_PIECE_Enemy_Gunfire)
	{
		bullet->delay = 0;
		bullet->speed = BULLET_STOP_SPEED;
		bullet->direction = DIR_None;
		bullet->active = 0;
	}
	else
	{
		bullet->delay = RandomNumber(100)+1;
		bullet->speed = BULLET_NORM_SPEED;
		bullet->active = 1;

		if (RandomNumber(2))
		{
			if (RandomNumber(2))
			{
				bullet->y = 0;
				bullet->direction = DIR_Down;
			}
			else
			{
				bullet->y = 88;
				bullet->direction = DIR_Up;
			}

			bullet->x = RandomNumber(160 - 8);
		}
		else
		{
			if (RandomNumber(2))
			{
				bullet->x = 0;
				bullet->direction = DIR_Right;
			}
			else
			{
				bullet->x = 152;
				bullet->direction = DIR_Left;
			}

			bullet->y = RandomNumber(92-4);
		}
	}
}

