// Header File
// Created 6/20/2010; 12:34:14 AM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_jumpman.h
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************


#ifndef _jm_jumpman_h
#define _jm_jumpman_h


#include "BasicTypes.h"

#include "jm_utilities.h"
#include "jm_levels.h"

uint1* pSprites1;
uint1* pSprites2;



#define GAME_AUTOTIMER    0x40000
#define TITLE_AUTOTIMER   0x40000


#define ANIM_STEP_MAX 16

typedef enum
{
	ACTION_None = 0,
	ACTION_Ladder_Still,
	ACTION_Walk,
	ACTION_Jump_Up,
	ACTION_Jump,
	ACTION_Ladder,
	//ACTION_Ladder_Still,
	ACTION_Chain,
	ACTION_Rope,
	ACTION_Fall,
	ACTION_Bounce,
	ACTION_Dead,
	ACTION_Shoot,
	ACTION_Total
} ACTION_Type;


typedef struct
{
	int2 x;
	int2 y;
	int2 x_orig;
	int2 y_orig;
	int2 x_freeze;
	int2 y_freeze;
	int2 x_jump;
	int2 y_jump;
	int2 anim_step_cur;
	int2 anim_step_max;
	int2 sprite_index;
	int2 sprite_index_freeze;
	//uint1* pSprite;
	//uint1* pSprite_freeze;
	DIR_Type direction;
	ACTION_Type action;
	SPECIAL_ABILITY_Type special_ability;
	uint1 bullet_index[4];
	uint1 bullet_fire_count;
	uint1 flags;
	uint1 freeze_count;
	//bool freeze;
	bool visible;
} SPRITE_Type;


typedef struct
{
	int2  x;
	int1  y;
	uint1 sprite_index;
} MOVE_History_Type;

typedef struct
{
	int2 anim_step_max;
	int1 sprite_index[ANIM_STEP_MAX];
	int1 x_offset[ANIM_STEP_MAX];
	int1 y_offset[ANIM_STEP_MAX];
} ANIM_Type;


// Global Variables
extern bool jumpman_dead;
extern int2 jumpmen_remaining;

extern SPRITE_Type jumpman;

#define MOVE_HISTORY_MAX   0x200
#define MOVE_HISTORY_MASK  0x1ff
extern  MOVE_History_Type move_history[MOVE_HISTORY_MAX];
extern int2 move_history_index;

extern const ANIM_Type action_anim_sequence[ACTION_Total];

// Prototypes
void _main(void);
void PlayGame(void);
void CheckKeyPress(void);
void DrawJumpman(int2 value, int2 x, int2 y);
void UpdateJumpman(void);
void SetJumpmanAction(ACTION_Type action, DIR_Type direction);
void SpawnJumpman(void);
void DrawFinalScore(void);
void UpdateJumpmanMoveHistory(void);

#endif
