// C Source File
// Created 11/21/2010; 1:06:31 PM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_objects.c
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************


#include <tigcclib.h>

#include "BasicTypes.h"

#include "jm_objects.h"
#include "jm_levels.h"


uint1 index_by_type = 0;
uint1 index_by_id   = 0;


OBJECT_Type* FindObjectEntryById(uint1 id, bool start_over)
{
	static uint1 index = 0;

	uint1 saved_index = index;
	bool found = false;
	bool first_time = true;
	int2 i;

	OBJECT_Type* object;

	if (start_over)
		object = &current_level.object_list[0];
	else
		object = &current_level.object_list[index];


	for (i = 0; i < OBJECTS_PER_LEVEL_MAX; i++)
	{
		if (object->id > 0)
		{
			if (object->id == id)
			{
				found = true;
				break;
			}

			object++;
			index++;
		}
		else
		{
			if (first_time)
			{
				first_time = false;
				index = 0;
				object = &current_level.object_list[index];
			}
			else
			{
				break;
			}
		}
	}

	if (!found)
		object = 0;

	if (start_over)
		index = saved_index;

	return object;
}


void ResetIndexByType(void)
{
	index_by_type = 0;
}


OBJECT_Type* GetNextObjectEntry(void)
{
	uint1 index = index_by_type;

	OBJECT_Type* object = &current_level.object_list[index];

	if ((object->id == 0) || (index >= OBJECTS_PER_LEVEL_MAX))
	{
		object = 0;
		index  = 0;
	}
	else
	{
		index++;
	}

	index_by_type = index;

	return object;
}


#if 1

OBJECT_Type* FindObjectEntryByType(GAME_PIECE_Type type, bool dummy)
{
	int2 i;
	uint1 index = index_by_type;
	bool found = false;
	bool local_dummy = dummy;

	OBJECT_Type* object = &current_level.object_list[index];

	for (i = 0; i < OBJECTS_PER_LEVEL_MAX; i++)
	{
		if (object->id > 0)
		{
			index++;

			if (object->game_piece == type)
			{
				found = true;
				break;
			}

			object++;
		}
		else
		{
			index = 0;
			object = &current_level.object_list[index];

			if (type != GAME_PIECE_Jumpman)
				break;
		}
	}

	if (!found)
		object = 0;
		
	index_by_type = index;

	return object;
}

#else

OBJECT_Type* FindObjectEntryByType(GAME_PIECE_Type type, bool start_over)
{
	static uint1 index = 0;

	uint1 saved_index = index;
	bool found = false;
	bool first_time = true;
	int2 i;

	OBJECT_Type* object;

	if (start_over)
		object = &current_level.object_list[0];
	else
		object = &current_level.object_list[index];


	for (i = 0; i < OBJECTS_PER_LEVEL_MAX; i++)
	{
		if (object->id > 0)
		{
			if (object->game_piece == type)
			{
				found = true;
				break;
			}

			object++;
			index++;
		}
		else
		{
			if (first_time)
			{
				first_time = false;
				index = 0;
				object = &current_level.object_list[index];
			}
			else
			{
				break;
			}
		}
	}

	if (!found)
		object = 0;

	if (start_over)
		index = saved_index;

	return object;
}

#endif
