// C Source File
// Created 11/21/2010; 6:41:55 AM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_platforms.c
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************


#include <tigcclib.h>
#include "extgraph.h"

#include "BasicTypes.h"
#include "GrayUtils.h"
#include "FontUtils.h"

#include "jm_platforms.h"
#include "jm_moving_platforms.h"
#include "jm_objects.h"
#include "jm_graphics.h"
#include "jm_jumpman.h"
#include "jm_levels.h"
#include "jm_collectables.h"
#include "jm_enemies.h"
#include "jm_timers.h"
#include "jm_bullets.h"

extern bool jump_in_progress;

void ConstructPlatformLevel(void)
{
	int2 length;
	int2 pass;
	int2 x;
	int2 y;

	OBJECT_Type* object;

	memset(plane_visible_0, 0xff, LCD_SIZE); // * 30
	memset(plane_visible_1, 0xff, LCD_SIZE); // * 30
	memset(plane_active_0,  0x00, LCD_SIZE); // * 30
	memset(plane_active_1,  0x00, LCD_SIZE); // * 30

	for (pass = 1; pass <= 2; pass++)
	{
		object = &current_level.object_list[0];

		while (object->id > 0)
		{
			if ((object->game_piece >= GAME_PIECE_Girder_Hor) &&
					(object->game_piece <= GAME_PIECE_Ladder))
			{
				if (((pass == 1) && (object->game_piece <= GAME_PIECE_Girder_Down)) ||
						((pass == 2) && (object->game_piece >  GAME_PIECE_Girder_Down)))// ||
						//((pass == 3) && (object->game_piece <= GAME_PIECE_Girder_Down))) // && (!(object->flags & FLAG_VISIBLE))))
				{
					x = object->x * 4;
					y = object->y;

					for (length = 0; length < object->length; length++)
					{
						DrawPlatformPiece(object->game_piece, object->flags, x, y);

						if (object->game_piece == GAME_PIECE_Girder_Hor)
						{
							x += 4;
						}
						else if (object->game_piece == GAME_PIECE_Girder_Up)
						{
							x += 4;
							y -= 1;
						}
						else if (object->game_piece == GAME_PIECE_Girder_Down)
						{
							x += 4;
							y += 1;
						}
						else
						{
							y += 4;
						}
					}
				}
			}

			object++;
		}
	}

	InitBullets();
	InitEnemies();
	InitMovingPlatforms();
	InitCollectables();
	InitTimers();
}

void Illuminate()
{
	if (jumpman.action != ACTION_None)
	{
		static uint2 active_piece_Z[14] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0};

		int2 i;
		int2 x;
		int2 y;

		uint2 active_piece_0[14];
		uint2 active_piece_1[14];
		uint2 active_piece_M0[14];
		uint2 active_piece_M1[14];

		y  = jumpman.y - 3;
		x  = jumpman.x - 4;

		if (x < 0)
			x = 0;

		if (y < 0)
			y = 0;

		Sprite16Get_R(x, y, 14, plane_active_0, active_piece_0);
		Sprite16Get_R(x, y, 14, plane_active_1, active_piece_1);

		for (i = 0; i < 14; i++)
		{
			active_piece_M0[i] = ~(active_piece_0[i]);
			active_piece_M1[i] = ~(active_piece_1[i]);
		}

		GraySprite16_SMASK_R(x, y, 14, active_piece_Z, active_piece_1, active_piece_M1, plane_visible_0, plane_visible_1);
		GraySprite16_SMASK_R(x, y, 14, active_piece_0, active_piece_Z, active_piece_M0, plane_visible_0, plane_visible_1);
	}
}

void Hotfoot()
{
	if ((jumpman.action == ACTION_Jump) || (jumpman.action == ACTION_Jump_Up))
	{
		if (jumpman.anim_step_cur < 3)
		{
		  uint1* pSprite = (uint1*)j3 + ((86 + jumpman.anim_step_cur*2) * (1*8*3));
		  
		  // draw the explosion
	  	GrayClipSprite8_SMASK_R(jumpman.x_jump - 4, jumpman.y_jump + 3, 8, pSprite +  0, pSprite +  8, pSprite + 16, PLANE_CurrentPlane0, PLANE_CurrentPlane1);
	  	GrayClipSprite8_SMASK_R(jumpman.x_jump + 4, jumpman.y_jump + 3, 8, pSprite + 24, pSprite + 32, pSprite + 40, PLANE_CurrentPlane0, PLANE_CurrentPlane1);
	  	
			if (jumpman.anim_step_cur == 1)
			{
				//uint1 clear_sprite[8] = {0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
				uint1 clear_sprite[8] = {0xff,0x7e,0xff,0xff,0xff,0xff,0x00,0x00};
				//uint1 clear_sprite[8] = {0xff,0x7e,0xff,0xff,0xff,0xff,0x00,0x00};
				
				// modify the active and visible planes to remove pixels
	  		//GrayClipSprite8_SMASK_R(jumpman.x_jump - 4, jumpman.y_jump + 3, 8, clear_sprite, clear_sprite, pSprite + 16, plane_visible_0, plane_visible_1);
	  		//GrayClipSprite8_SMASK_R(jumpman.x_jump + 4, jumpman.y_jump + 3, 8, clear_sprite, clear_sprite, pSprite + 40, plane_visible_0, plane_visible_1);
	  		//GrayClipSprite8_SMASK_R(jumpman.x_jump - 4, jumpman.y_jump + 3, 8, clear_sprite, clear_sprite, pSprite + 16, plane_active_0, plane_active_1);
	  		//GrayClipSprite8_SMASK_R(jumpman.x_jump + 4, jumpman.y_jump + 3, 8, clear_sprite, clear_sprite, pSprite + 40, plane_active_0, plane_active_1);
	  		//GrayClipSprite8_OR_R(jumpman.x_jump - 4, jumpman.y_jump + 3, 8,  pSprite + 16, pSprite + 16, plane_visible_0, plane_visible_1);
	  		//GrayClipSprite8_OR_R(jumpman.x_jump + 4, jumpman.y_jump + 3, 8,  pSprite + 40, pSprite + 40, plane_visible_0, plane_visible_1);
	  		//GrayClipSprite8_AND_R(jumpman.x_jump - 4, jumpman.y_jump + 3, 8, pSprite + 16, pSprite + 16, plane_active_0, plane_active_1);
	  		//GrayClipSprite8_AND_R(jumpman.x_jump + 4, jumpman.y_jump + 3, 8, pSprite + 40, pSprite + 40, plane_active_0, plane_active_1);
	  		GrayClipSprite8_OR_R (jumpman.x_jump, jumpman.y_jump + 7, 2, clear_sprite, clear_sprite, plane_visible_0, plane_visible_1);
	  		GrayClipSprite8_AND_R(jumpman.x_jump, jumpman.y_jump + 7, 2, clear_sprite+6, clear_sprite+6, plane_active_0,  plane_active_1);
	  	
			}
	  			
		}
	}
}

void DrawPlatformPiece(uint1 game_piece, uint1 flags, int2 x, int2 y)
{
	uint1* pSprite = (uint1*)j7 + (game_piece * (1*4*3));

	if ((flags & (FLAG_VISIBLE | FLAG_MOVABLE)) == FLAG_VISIBLE)
	{
		GraySprite8_SMASK_R(x, y, 4, pSprite, pSprite+4, pSprite+8, plane_visible_0, plane_visible_1);
	}

	if ((flags & (FLAG_ACTIVE | FLAG_MOVABLE)) == FLAG_ACTIVE)
	{
		if (game_piece <= GAME_PIECE_Rope_Up)
			Sprite8_OR_R(x, y, 4, pSprite+4, plane_active_1);
		else
			Sprite8_OR_R(x, y, 4, pSprite, plane_active_0);
	}
}

void ModifyPlatformPiece(uint1 game_piece, uint1 new_flags, int2 x, int2 y)
{
	uint1* p_orig_platform_sprite = (uint1*)j7 + (game_piece * (1*4*3));

	if (new_flags & FLAG_VISIBLE)
	{
		// Note: ladders and ropes have visual priority
		if (game_piece <= GAME_PIECE_Rope_Up)
		{
			// it is possible here to draw a girder/chain over a ladder
			uint4 active_piece_0;
			uint4 active_piece_1;

		Sprite8Get_R(x, y, 4, plane_active_0, (uint1*)&active_piece_0);
		
		//active_piece_1[0] = ~active_piece_0[0];
		//active_piece_1[1] = ~active_piece_0[1];
		//active_piece_1[2] = ~active_piece_0[2];
		//active_piece_1[3] = ~active_piece_0[3];
		active_piece_1 = ~active_piece_0;
		
		//Sprite8Get_R(x, y, 4, plane_active_1, active_piece_1);
			
			// first draw the girder/chain
			Sprite8_AND_R(x, y, 4, p_orig_platform_sprite+8, plane_visible_0);
			Sprite8_OR_R(x, y, 4, p_orig_platform_sprite+4, plane_visible_1);

			// now restore the ladder/rope plane
		//Sprite8Get_R(x, y, 4, plane_active_0, active_piece_0);
			Sprite8_AND_R(x, y, 4, (uint1*)&active_piece_1, plane_visible_1);
			Sprite8_OR_R(x, y, 4, (uint1*)&active_piece_0, plane_visible_0);
		}
		else if (game_piece <= GAME_PIECE_Ladder)
		{
			Sprite8_AND_R(x, y, 4, p_orig_platform_sprite+8, plane_visible_1);
			Sprite8_OR_R(x, y, 4, p_orig_platform_sprite, plane_visible_0);
		}
	}
	else
	{
		if (game_piece <= GAME_PIECE_Rope_Up)
		{
			//Sprite8_AND_R(x, y, 4, p_orig_platform_sprite+8, plane_visible_0);
			Sprite8_OR_R(x, y, 4, p_orig_platform_sprite+4, plane_visible_0);
		}
		else if (game_piece <= GAME_PIECE_Ladder)
		{
			// it is possible here to erase a ladder/rope over a girder/chain
			//uint1 active_piece_1[4];
			//uint4* active_piece_0 = (uint4*)&active_piece_1;
			uint4 active_piece_1;
			//uint4* active_piece_0 = (uint4*)&active_piece_1;
			
			// first erase the ladder/rope
		  //Sprite8_AND_R(x, y, 4, p_orig_platform_sprite+8, plane_visible_1);
			Sprite8_OR_R(x, y, 4, p_orig_platform_sprite, plane_visible_1);

			// now restore the girder/chain
			Sprite8Get_R(x, y, 4, plane_active_1, (uint1*)&active_piece_1);
			Sprite8_OR_R(x, y, 4, (uint1*)&active_piece_1, plane_visible_1);

			//active_piece_1[0] = ~active_piece_1[0];
			//active_piece_1[1] = ~active_piece_1[1];
			//active_piece_1[2] = ~active_piece_1[2];
			//active_piece_1[3] = ~active_piece_1[3];

			//*active_piece_0 = ~(*active_piece_0);
			active_piece_1 = ~(active_piece_1);

			Sprite8_AND_R(x, y, 4, (uint1*)&active_piece_1, plane_visible_0);
			//Sprite8_AND_R(x, y, 4, (uint1*)active_piece_0, plane_visible_0);
			//Sprite8_AND_R(x, y, 4, active_piece_1, plane_visible_0);
		}
	}

	if (new_flags & FLAG_ACTIVE)
	{
		// Note: active planes do not have visual priorities
		if (game_piece <= GAME_PIECE_Rope_Up)
		{
			// draw the girder/chain to active plane 1
			Sprite8_AND_R(x, y, 4, p_orig_platform_sprite+8, plane_active_1);
			Sprite8_OR_R(x, y, 4, p_orig_platform_sprite+4, plane_active_1);
		}
		else if (game_piece <= GAME_PIECE_Ladder)
		{
			// draw the ladder/rope to active plane 0
			Sprite8_AND_R(x, y, 4, p_orig_platform_sprite+8, plane_active_0);
			Sprite8_OR_R(x, y, 4, p_orig_platform_sprite, plane_active_0);
		}
	}
	else
	{
		if (game_piece <= GAME_PIECE_Rope_Up)
		{
			// clear the girder/chain to active plane 1
			Sprite8_AND_R(x, y, 4, p_orig_platform_sprite+8, plane_active_1);
			//Sprite8_OR_R(x, y, 4, p_orig_platform_sprite+4, plane_active_0);
		}
		else if (game_piece <= GAME_PIECE_Ladder)
		{
			// clear the ladder/rope to active plane 0
			Sprite8_AND_R(x, y, 4, p_orig_platform_sprite+8, plane_active_0);
			//Sprite8_OR_R(x, y, 4, p_orig_platform_sprite, plane_active_1);
		}
	}
}

void DrawPlatformLevel(void)
{
	//memcpy(PLANE_CurrentPlane0, PLANE_visible_0, LCD_SIZE);
	//memcpy(PLANE_CurrentPlane1, plane_visible_1, LCD_SIZE);
	FastCopyScreen_R(plane_visible_0, PLANE_CurrentPlane0);
	FastCopyScreen_R(plane_visible_1, PLANE_CurrentPlane1);
}


bool girder_check_row[12];
bool ladder_check_row[12];
bool ladder_check = 0;
bool chain_check = 0;
bool rope_check = 0;
int2 ladder_rung_index = 0;
int2 skip_rope_check = 0;
int2 skip_chain_check = 0;


void CheckPlatformInteractions(DIR_Type direction)
{
	ladder_check = false;
	chain_check  = false;
	rope_check   = false;

	memset (girder_check_row, 0, 12);
	memset (ladder_check_row, 0, 12);

//	if ((jumpman.x >= 0) && (jumpman.x <= 152))
	if ((jumpman.x > -4) && (jumpman.x <= 156))
	{
		int2   i;
		int2   jumpman_x_adjust = 0;
		//int2   jumpman_y_adjust = 0;
		bool   walk_not_in_progress = false;
		bool   jump_not_in_progress = false;
		bool   death_not_in_progress = false;
		uint1  test_value;
		uint1  rung_value = 0;

		if ((jumpman.action < ACTION_Fall) || (jumpman.action > ACTION_Dead))
			death_not_in_progress = true;

		if (jumpman.action != ACTION_Jump)
			jump_not_in_progress = true;

		if (jumpman.action != ACTION_Walk)
			walk_not_in_progress = true;

		if (death_not_in_progress)
		{
			// check for ladder interactions by finding a ladder rung
			for (i = 0; i <= 11; i++)
			{
				test_value = *(jumpman_bkgd_0 + i);

				// test for jumpman within 2 pixels from a ladder
				if ((test_value == 0xff) ||
						(jump_not_in_progress && ((test_value == 0x3f) || (test_value == 0xfc))))
				{
					// store the pixel value of the rung
					ladder_check = true;
					rung_value = test_value;
					break;
				}
			}

			if (ladder_check)
			{
				for (i = 2; i <= 8; i++)
				{
					test_value = *(jumpman_bkgd_0 + i) & rung_value;

					//if ((test_value & 0xc3) == 0xc3)//0x00)
					if ((test_value == 0xc3) || (test_value == 0xff))
					{
						//GrayUserDrawStr(74,1, "Ladder", COLOR_White, 0);
						ladder_check_row[i] = true;
					}
					else if (jump_not_in_progress)
					{
						// if a jump is not in progress, test for jumpman within 2 pixels on ladder
						if ((test_value == 0x0c) || (test_value == 0xfc))
						{
							if ((direction == DIR_Up) || (direction == DIR_Down))
							  jumpman_x_adjust = -2;

							//GrayUserDrawStr(74,1, "Ladder -2", COLOR_White, 0);
							ladder_check_row[i] = true;
						}
						else if ((test_value == 0x30) || (test_value == 0x3f))
						{
							if ((direction == DIR_Up) || (direction == DIR_Down))
								jumpman_x_adjust = +2;

							ladder_check_row[i] = true;
							//GrayUserDrawStr(74,1, "Ladder +2", COLOR_White, 0);
						}
					}
				}
			}
			else
			{
				for (i = 1; i <= 5; i += 4)
				{
					test_value = *(jumpman_bkgd_1 + i);

					if ((test_value == 0x08) || (test_value == 0x10))
					{
						chain_check = true;
						break;
					}
					else if (jump_not_in_progress && walk_not_in_progress)
					{
						if ((test_value == 0x02) || (test_value == 0x04))
						{
							jumpman_x_adjust = 2;
							chain_check = true;
							break;
						}
						else if ((test_value == 0x20) || (test_value == 0x40))
						{
							jumpman_x_adjust = -2;
							chain_check = true;
							break;
						}
					}
				}

				if (!chain_check)
				{
					// Analyze rope interactions
					for (i = 0; i <= 6; i += 6)
					{
						test_value = *(jumpman_bkgd_0 + 7 - i);

						if ((test_value == 0x08) || (test_value == 0x10))
						{
							rope_check = true;
							//GrayUserDrawStr(74,1, "Rope +0", COLOR_White, 0);
							break;
						}
						else if (jump_not_in_progress && walk_not_in_progress)
						{
							if ((test_value == 0x02) || (test_value == 0x04))
							{
								jumpman_x_adjust = 2;
								rope_check = true;
								//GrayUserDrawStr(74,1, "Rope +2", COLOR_White, 0);
								break;
							}
							else if ((test_value == 0x20) || (test_value == 0x40))
							{
								jumpman_x_adjust = -2;
								rope_check = true;
								//GrayUserDrawStr(74,1, "Rope -2", COLOR_White, 0);
								break;
							}
						}
					}
				}
			}
		}

		// Analyze girder interactions
//der		for (i = 1; i <= 10; i++)
		for (i = 1; i <= 11; i++)
		{
			//uint2 both_planes = (*test_address_0) & (*test_address_1);
			//test_value = (*test_address_1) & (~both_planes);
			//test_value = (*(jumpman_bkgd_1 + i)) & (~(*(jumpman_bkgd_0 + i)));
			test_value = *(jumpman_bkgd_1 + i);

			if (((test_value & 0x18) == 0x18) ||
					((test_value & 0x30) == 0x30) ||
					((test_value & 0x0c) == 0x0c))
			{
				girder_check_row[i] = true;
			}
			else if (ladder_check == true)
			{
				if ((direction == DIR_Left) || (direction == DIR_Right))
				{
					//test_value = (*(jumpman_bkgd_1 + i + 1)) & (~(*(jumpman_bkgd_0 + i + 1)));
					test_value = *(jumpman_bkgd_1 + i + 1);

					if (((test_value & 0xa0) == 0xa0) ||
							((test_value & 0x0a) == 0x0a) ||
							((test_value & 0x28) == 0x28))
						girder_check_row[i] = true;
				}
			}
		}

		jumpman.x += jumpman_x_adjust;

	}
}

bool CheckJumpmanLadderRow(int2 y1, int2 y2)
{
	int2 i;

	for (i = y1; i <= y2; i++)
	{
		if (ladder_check_row[i])
			return true;
	}

	return false;
}

bool CheckJumpmanGirderRow(int2 y1, int2 y2)
{
	int2 i;

	for (i = y1; i <= y2; i++)
	{
		if (girder_check_row[i])
			return true;
	}

	return false;
}


bool CheckJumpmanLadder(void)
{
	return CheckJumpmanLadderRow(2, 8);
}

bool CheckJumpmanChain(void)
{
	return chain_check;
}

bool CheckJumpmanRope(void)
{
	return rope_check;
}

