// C Source File
// Created 1/28/2011; 11:44:34 PM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_scores.c
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************


#include <tigcclib.h>
#include "extgraph.h"

#include "BasicTypes.h"
#include "GrayUtils.h"
#include "FontUtils.h"
#include "TimeUtils.h"
#include "BasicUtils.h"
#include "KeyUtils.h"
#include "FileUtils.h"

#include "jm_scores.h"
#include "jm_jumpman.h"
#include "jm_graphics.h"

int4 game_score;
uint4 bonus_points_total;
uint2 bonus_points_level;
uint2 bonus_points_jumpman;

int2 selected_index_top_scores;
int2 selected_index_high_bonus;

SCORE_ENTRY_Type* high_score_entry[2];

bool scores_inserted;

int2 name_string_index = 0;
char name_string[4] = "   ";


//HIGH_SCORES_Type high_scores_list;
//HIGH_SCORES_Type high_scores_list_file =
HIGH_SCORES_Type high_scores_list =
{
#if 0
	{{
		{"XYZ",25000},
		{"JIM",22500},
		{"ABC",20000},
		{"BOB",17500},
		{"DER",15000},
		{"JIM",12500},
		{"XYZ",10000},
		{"JIM",7500},
		{"ABC",5000},
		{"BOB",2500},
		{"   ",0},
	},
	{
		{"JIM",15000},
		{"BOB",13500},
		{"DER",12000},
		{"XYZ",10500},
		{"ABC",9000},
		{"JIM",7500},
		{"BOB",6000},
		{"DER",4500},
		{"XYZ",3000},
		{"ABC",1500},
		{"   ",0},
	},},
#else
	{{
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
	},
	{
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
		{"   ",0},
	},},
#endif

	0x00000000,

	{ 0, 0, 0,'j','m','s', 0, OTH_TAG}

};

void ReadHighScores(void)
{
	const char file_name[] = "jms";
	uint4 file_address;
	
	file_address = (uint4)OpenFile((char*)file_name);

	if (file_address)
	{
		memcpy(&high_scores_list, (void*)file_address, (uint4)sizeof(HIGH_SCORES_Type));
	}
}

void SaveHighScores(void)
{
	const char file_name[] = "jms";
	uint4 file_address;
	
	UnarchiveFile((char*)file_name);
	
	file_address = (uint4)OpenFile((char*)file_name);

	if (file_address)
	{
		memcpy((void*)file_address, &high_scores_list, (uint4)sizeof(HIGH_SCORES_Type));
	}
	else
	{
		NewFile((char*)file_name, &high_scores_list, sizeof(HIGH_SCORES_Type));
	}

	CloseFile((char*)file_name);
	
	ArchiveFile((char*)file_name);
}

void DrawFinalScores(void)
{
	char buffer[32];
	const char line_12[] = "~~~~~~~~~~~~";

	GrayUserDrawStr(80-(11*4/2),20, "TOTAL SCORE", COLOR_White, 0x00);
	GrayUserDrawStr(80-(12*4/2),31, (char*)line_12, COLOR_White, 0x00);
	GrayUserDrawStr(80-(12*4/2),33, (char*)line_12, COLOR_White, 0x00);
	GrayUserDrawStr(80-(11*4/2),50, "TOTAL BONUS", COLOR_White, 0x00);
	GrayUserDrawStr(80-(12*4/2),61, (char*)line_12, COLOR_White, 0x00);
	GrayUserDrawStr(80-(12*4/2),63, (char*)line_12, COLOR_White, 0x00);

	sprintf(buffer, "%07lu", game_score);
	GrayUserDrawStr(80-(7*4/2),32, buffer, COLOR_Black, 0x00);
	sprintf(buffer, "%07lu", bonus_points_total);
	GrayUserDrawStr(80-(7*4/2),62, buffer, COLOR_Black, 0x00);

}

void GetHighScoreName(void)
{
	int2 i;

	if (scores_inserted)
	//if ((scores_inserted) && ((frame_count & 0x2) == 0x2))
	{
		KeyboardInput key_pressed = GetKeyPress(false);

		for (i = 0; i < 2; i++)
		{
			if (high_score_entry[i])
			{
				memcpy(high_score_entry[i]->name, name_string, 3);
			}
		}

		if (key_pressed == KBD_Backspace)
		{
			if (name_string_index > 0)
			{
				name_string_index--;
				name_string[name_string_index] = ' ';
			}
		}
		else if (key_pressed == KBD_Clear)
		{
			while (name_string_index > 0)
			{
				name_string_index--;
				name_string[name_string_index] = ' ';
			}
		}
		else if (key_pressed == KBD_Enter)
		{
			scores_inserted = false;
			memset (name_string, ' ', 3);
			name_string_index = 0;
		}
		else if ((key_pressed >= KBD_A) && (key_pressed <= KBD_Z))
		{
			if (name_string_index <= 2)
			{
				name_string[name_string_index] = key_pressed;
				name_string_index++;
			}
		}
	}
}

void InsertHighScores(uint4 game_score, uint4 bonus_score)
{
	static const SCORE_ENTRY_Type empty_score_entry = {"   ", 0};
	SCORE_ENTRY_Type* score_entry = 0;
	uint4 score[2] = {game_score, bonus_score};
	int2 i;
	int2 j;

	scores_inserted = false;

	for (i = 0; i < 2; i++)
	{
		high_score_entry[i] = 0;

		score_entry = &high_scores_list.high_scores[i][9];

		for (j = 0; j < 10; j++)
		{
			//score_entry = &high_scores_list.high_scores[i][j];

			if (score[i] > score_entry->score)
			{
				// move this score entry down one entry
				memcpy (score_entry + 1, score_entry, sizeof(SCORE_ENTRY_Type));

				// initialize this entry with an empty score entry
				memcpy (score_entry, &empty_score_entry, sizeof(SCORE_ENTRY_Type));

				// populate the score field, the name will be filled in later
				score_entry->score = score[i];

				high_score_entry[i] = score_entry;
				scores_inserted = true;
			}
			else
			{
				break;
			}

			score_entry--;
		}
	}
}

bool DrawHighScores(void)
{
	int2 i;
	int2 j;
	int2 x;
	int2 y;
	char hs_buffer[40];
	uint2 select_mask = 0xf0;
	SCORE_ENTRY_Type* top_scores;
	SCORE_ENTRY_Type* high_bonus;
	SCORE_ENTRY_Type* score_entry;

 	GrayFillScreen(COLOR_Black);

	DrawWhiteLine(  1,  1, 78,  1);
	DrawWhiteLine( 78,  1, 78, 98);
	DrawWhiteLine( 78, 98,  1, 98);
	DrawWhiteLine(  1, 98,  1,  1);
	DrawWhiteLine(  1, 11, 78, 11);

	GrayUserDrawStr(1+19,3, "TOP SCORES", COLOR_White, 0x00);

	DrawWhiteLine( 81,  1,158,  1);
	DrawWhiteLine(158,  1,158, 98);
	DrawWhiteLine(158, 98, 81, 98);
	DrawWhiteLine( 81, 98, 81,  1);
	DrawWhiteLine( 81, 11,158, 11);

	GrayUserDrawStr(81+19,3, "HIGH BONUS", COLOR_White, 0x00);

	top_scores = &high_scores_list.high_scores[0][0];
	high_bonus = &high_scores_list.high_scores[1][0];

	GetHighScoreName();

	x = 1 + 3;

	for (i = 0; i < 2; i++)
	{
		score_entry = &high_scores_list.high_scores[i][0];

		y = 16;

		for (j = 0; j < 10; j++)
		{
			if ((scores_inserted) && (score_entry == high_score_entry[i]))
			{
				select_mask = 0xf0;
				//DrawWhiteLine( x, y-1, x+71,  y-1);
			}
			else
			{
				select_mask = 0;
			}

//			sprintf(hs_buffer, " %02d. %s  %07lu ", j+1, score_entry->name, score_entry->score);
			sprintf(hs_buffer, " %2d. %s  %7lu ", j+1, score_entry->name, score_entry->score);
			GrayUserDrawStr(x,y, hs_buffer, COLOR_White, select_mask);

			if ((scores_inserted) && ((frame_count & 0x2) == 0x2))
			{
				GrayUserDrawStr(x + (5 + name_string_index) * 4, y, "_", COLOR_Black, select_mask);
			}

			score_entry++;

			y += 8;
		}

		x += 80;
	}
	
	return scores_inserted;
}