// C Source File
// Created 1/22/2011; 11:34:29 PM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_titlescreen.c
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************



#include <tigcclib.h>
#include "extgraph.h"

#include "BasicTypes.h"
#include "GrayUtils.h"
#include "FontUtils.h"
#include "TimeUtils.h"
#include "BasicUtils.h"

#include "jm_titlescreen.h"
#include "jm_jumpman.h"
#include "jm_graphics.h"
#include "jm_objects.h"
#include "jm_levels.h"
#include "jm_scores.h"
#include "jm_platforms.h"


// Titlescreen
OBJECT_Type titlescreen_object_list[13] =
{
	{ 1, GAME_PIECE_Jumpman,    FLAG_VA, 19, 20,  SPECIAL_ABILITY_None},

	{ 1, GAME_PIECE_Girder_Hor, FLAG_VA,  0, 28,  40},
	{ 1, GAME_PIECE_Girder_Hor, FLAG_VA,  0, 88,  40},

	{23, GAME_PIECE_Ladder,     FLAG_VA,  1, 24,  16},
	{24, GAME_PIECE_Ladder,     FLAG_VA, 37, 24,  16},

	{25, GAME_PIECE_Rope_Up,    FLAG_VA,  4, 28, 10},
	{25, GAME_PIECE_Rope_Up,    FLAG_VA,  9, 28,  8},
	{25, GAME_PIECE_Rope_Up,    FLAG_VA, 14, 28,  6},
	{25, GAME_PIECE_Rope_Up,    FLAG_VA, 19, 28,  6},
	{25, GAME_PIECE_Rope_Up,    FLAG_VA, 24, 28,  6},
	{25, GAME_PIECE_Rope_Up,    FLAG_VA, 29, 28,  6},
	{25, GAME_PIECE_Rope_Up,    FLAG_VA, 34, 28,  6},

	{ 0, 0, 0, 0, 0, 0},
};


LETTER_Type  letter_list_orig[7] =
{
	{156,18,1,1},
	{156,18,2,2},
	{156,18,3,0},
	{156,18,4,1},
	{156,18,3,2},
	{156,18,5,0},
	{156,18,6,1},
};


SCRIPTED_MOVE_Type  scripted_move_list[] =
{
	{38, DIR_Right, false},
	//{1, DIR_None,  false},
	{74, DIR_Left,  false},
	{74, DIR_Right, false},
	{64, DIR_Left,  false},
	{64, DIR_Right, false},
	{54, DIR_Left,  false},
	{54, DIR_Right, false},
	{44, DIR_Left,  false},
	{44, DIR_Right, false},
	{34, DIR_Left,  false},
	{34, DIR_Right, false},
	{24, DIR_Left,  false},

	{24, DIR_Right, false},
	{14, DIR_Left,  false},
	{14, DIR_None,  false},
	{14, DIR_Up,    true},
	{14, DIR_Right, false},
	{14, DIR_Left,  false},
	{14, DIR_None,  false},
	{14, DIR_Up,    true},
	{14, DIR_Right, false},
	{39, DIR_Left,  false},
	{39, DIR_Right, false},
	{14, DIR_Left,  true},
	{25, DIR_Left,  false},
	{99, DIR_None,  false},
	{14, DIR_Up,    true},
	{0,0,0},
};

SCRIPTED_MOVE_Type  letter_move_list[] =
{
	{38, DIR_None,  false},
	{72, DIR_Left,  false},
	{17, DIR_Down,  false},
	{ 1, DIR_None,  true},

	{58, DIR_None,  false},
	{62, DIR_Left,  false},
	{17, DIR_Down,  false},
	{ 1, DIR_None,  true},

	{48, DIR_None,  false},
	{52, DIR_Left,  false},
	{17, DIR_Down,  false},
	{ 1, DIR_None,  true},

	{38, DIR_None,  false},
	{42, DIR_Left,  false},
	{17, DIR_Down,  false},
	{ 1, DIR_None,  true},

	{28, DIR_None,  false},
	{32, DIR_Left,  false},
	{17, DIR_Down,  false},
	{ 1, DIR_None,  true},

	{18, DIR_None,  false},
	{22, DIR_Left,  false},
	{17, DIR_Down,  false},
	{ 1, DIR_None,  true},

	{ 8, DIR_None,  false},
	{14, DIR_Left,  false},
	{14, DIR_Right, false},
	{28, DIR_None,  false},
	{14, DIR_Left,  false},
	{14, DIR_Right, false},
	{28, DIR_None,  false},
	{49, DIR_Left,  false},
	{37, DIR_Right, false},
	{17, DIR_Down,  false},
	{ 1, DIR_None,  true},

	{120, 100,  false},
	{1, 101,  false},
	{1, 102,  false},
	{8, DIR_None,  false},
	{1, 102,  false},
	{8, DIR_None,  false},
	{1, 102,  false},
	{8, DIR_None,  false},
	{1, 102,  false},
	{8, DIR_None,  false},
	{1, 102,  false},
	{8, DIR_None,  false},
	{1, 102,  false},
	{8, DIR_None,  false},
	{1, 102,  false},
	{1, 103,  false},

	{0,0,0},
};

LETTER_Type* letter_in_motion;
LETTER_Type  letter_list[7];
SCRIPTED_MOVE_Type* scripted_move;
SCRIPTED_MOVE_Type* letter_move;
uint2 scripted_move_count;
uint2 letter_move_count;
bool titlescreen;

int2 DrawTitleScreen(void)
{
	bool done = false;
	int2 choice = 0;

	titlescreen = true;
	jumpman_dead = false;
	reset_game = false;
	frame_count = 0;
	game_stage_count = 0;

	EraseScreen();
	ResetAutoQuit();
	InitTitleScreen();
	SetGameStage(STAGE_display_title);

	while (!done)
	{
		CheckExitGame();
		CheckResetGame();
		CheckAdjustContrast();
		CheckAutoQuit(TITLE_AUTOTIMER);

		switch (game_stage)
		{
			case STAGE_display_title:

				if (jumpman_dead)
				{
					jumpman_dead = false;

					SetGameStage(STAGE_high_scores);
				}

				DrawPlatformLevel();

				CheckLetters();

				UpdateJumpman();
				CheckKeyPress();

				GrayUserDrawStr(9*4, 1, "DAVID RANDALL PRESENTS", COLOR_White, 0);
				GrayUserDrawStr(2*4,92, "RANDY GLOVER'S JUMPMAN FOR THE TI-89", COLOR_White, 0);
				GrayUserDrawStr((35*4)/2,80, "v1.01", COLOR_DGray, 0);

				break;

			case STAGE_high_scores:

				DrawHighScores();

				if (game_stage_count > 100)
				{
					InitTitleScreen();
					SetGameStage(STAGE_display_title);
				}

				break;

			default:
				break;
		}

		if (game_stage_count > 10)
		{
			if (_keytest (RR_2ND))
			{
				while (_keytest (RR_2ND));
				choice = 1;
				done = true;
			}
		}

		if (exit_game || reset_game)
		{
			done = true;
		}

		//frame_count++;
		game_stage_count++;

		Sleep(4);
		SwapPlaneBuffers();
	}

	return choice;
}

void InitTitleScreen(void)
{
	memset(&current_level, 0, sizeof(LEVEL_Type));

	memcpy(current_level.object_list, titlescreen_object_list, sizeof(OBJECT_Type) * 13);
	memcpy(letter_list, letter_list_orig, sizeof(LETTER_Type) * 7);

	scripted_move = &scripted_move_list[0];
	scripted_move_count = 0;

	letter_move = &letter_move_list[0];
	letter_move_count = 0;

	letter_in_motion = &letter_list[0];

	ConstructPlatformLevel();
	//DrawPlatformLevel();

	SpawnJumpman();
	UpdateJumpman();
}


void GetScriptedMove(DIR_Type* direction, bool* action)
{
	bool done = false;

	*direction = DIR_None;
	*action = false;

	while(!done)
	{
		if (scripted_move->count == 0)
		{
			*direction = DIR_None;
			*action = false;
			scripted_move_count = 0;
			done = true;
		}
		else if (scripted_move_count < scripted_move->count)
		{
			*direction = (DIR_Type)scripted_move->direction;
			*action = scripted_move->action;
			scripted_move_count++;
			done = true;

			//if (*direction == DIR_Left)
			//	GrayUserDrawStr(70, 20, "LEFT", COLOR_White, 0);
			//else if (*direction == DIR_Right)
			//	GrayUserDrawStr(70, 20, "RIGHT", COLOR_White, 0);
		}
		else
		{
			scripted_move++;
			scripted_move_count = 0;
			//GrayUserDrawStr(70, 20, "HIT ME", COLOR_White, 0);
		}
	}
}

void GetLetterMove(DIR_Type* direction, bool* action)
{
	bool done = false;

	while(!done)
	{
		if (letter_move->count == 0)
		{
			*direction = DIR_None;
			*action = false;
			letter_move_count = 0;
			done = true;
		}
		else if (letter_move_count < letter_move->count)
		{
			*direction = (DIR_Type)letter_move->direction;
			*action = letter_move->action;
			letter_move_count++;
			done = true;
		}
		else
		{
			letter_move++;
			letter_move_count = 0;
		}
	}
}

void CheckLetters(void)
{
	DIR_Type direction = DIR_None;
	bool next_letter = false;
	int2 i;
	int2 color_increment = 0;

	GetLetterMove(&direction, &next_letter);

	if (next_letter)
	{
		letter_in_motion++;
	}
	else if (direction == DIR_Left)
	{
		letter_in_motion->x -= 2;
	}
	else if (direction == DIR_Down)
	{
		letter_in_motion->y += 2;
	}
	else if (direction == DIR_Right)
	{
		letter_in_motion->x += 2;
	}
	else if (direction == DIR_Up)
	{
		letter_in_motion->y -= 2;
	}
	else if (direction == 100)
	{
		color_increment = 1;
	}
	else if (direction == 101)
	{
		letter_in_motion = &letter_list[0];
	}
	else if (direction == 102)
	{
		letter_in_motion->color = 3;
		letter_in_motion++;
	}
	else if (direction == 103)
	{
		jumpman_dead = true;
	}

	for (i = 0; i < 7; i++)
	{
		LETTER_Type* letter = &letter_list[i];

		if ((color_increment) && ((game_stage_count & 0x3) == 0x3) )
		{
			letter->color += color_increment;

			if (letter->color >= 3)
				letter->color = 0;
		}

		DrawJumpmanLetter(letter->letter, letter->color, letter->x, letter->y);
	}
}

