// C Source File
// Created 1/29/2011; 1:17:45 PM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_utilites.c
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************


#include <tigcclib.h>

#include "jm_utilities.h"

bool exit_game;
bool reset_game;
int2 game_speed;
uint4 frame_count;
uint4 game_stage_count;
STAGE_Type game_stage;

uint4 auto_quit_count;



void GetPlayerKeyPress(DIR_Type* direction, bool* action)
{
	static DIR_Type previous_direction = DIR_None;

	if (_keytest (RR_UP))
	{
		if (_keytest (RR_LEFT))
			*direction = DIR_UpLeft;
		else if (_keytest (RR_RIGHT))
			*direction = DIR_UpRight;
		else
			*direction = DIR_Up;
	}
	else if (_keytest (RR_DOWN))
	{
		if (_keytest (RR_LEFT))
			*direction = DIR_DownLeft;
		else if (_keytest (RR_RIGHT))
			*direction = DIR_DownRight;
		else
			*direction = DIR_Down;
	}
	else if (_keytest (RR_LEFT))
	{
		*direction = DIR_Left;
	}
	else if (_keytest (RR_RIGHT))
	{
		*direction = DIR_Right;
	}
	else
	{
		*direction = DIR_None;
	}

	if (_keytest (RR_2ND))
	{
		*action = true;
	}
	else
	{
		*action = false;
	}

	// if the direction key is being held down, then simply return no direction key pressed
	previous_direction = *direction;
}

void SetGameStage(STAGE_Type new_game_stage)
{
	game_stage = new_game_stage;
	game_stage_count = 0;
}

bool AdjustGameSpeed(void)
{
	int2 game_speed_temp = game_speed;
	bool game_speed_adjusted = false;

	if (_keytest (RR_F1))
		game_speed_temp = 1;
	else if (_keytest (RR_F2))
		game_speed_temp = 2;
	else if (_keytest (RR_F3))
		game_speed_temp = 3;
	else if (_keytest (RR_F4))
		game_speed_temp = 4;
	else if (_keytest (RR_F5))
		game_speed_temp = 5;

	if (game_speed_temp)
	{
		game_speed = game_speed_temp;
		game_speed_adjusted = true;
	}

	return game_speed_adjusted;
}

void CheckExitGame(void)
{
	if (_keytest (RR_ESC))
	{
		// wait until key is released
		while (_keytest (RR_ESC));
		exit_game = true;
	}
}

void CheckResetGame(void)
{
	if (_keytest (RR_APPS))
	{
		// wait until key is released
		while (_keytest (RR_APPS));
		reset_game = true;
	}
}

void CheckAdjustContrast(void)
{
	if (_keytest (RR_MINUS))
	{
		// wait until key is released
		while (_keytest (RR_MINUS));
		OSContrastDn();
		ResetAutoQuit();
	}
	else if (_keytest (RR_PLUS))
	{
		// wait until key is released
		while (_keytest (RR_PLUS));
		OSContrastUp();
		ResetAutoQuit();
	}
}

void CheckAutoQuit(uint4 value)
{
	if (++auto_quit_count >= value)
	{
		exit_game = true;
	}
}

void ResetAutoQuit(void)
{
	auto_quit_count = 0;
}

uint2 CalculateVpc(uint2* address, int2 size)
{
	uint2 vpc = 0;
	int2 i;

	size = size >> 1;

	for (i = 0; i < size; i++)
	{
		vpc = vpc ^ (*address);
	}

	return vpc;
}

bool IsVpcValid(uint2* address, int2 size, uint2 vpc)
{
	bool vpc_valid = false;

	if (vpc == CalculateVpc(address, size))
		vpc_valid = true;

	return vpc_valid;
}

